const express = require("express");
var bodyParser = require('body-parser')
const passport = require("passport");
const initializePassportCustomer = require("./passportCustomerConfig");
const initializePassport = require("./passportAdminConfig");
const session = require("express-session");
const flash = require("express-flash");
var crypto = require("crypto");
const moment = require("moment");
const multer = require("multer");


const { pool } = require("./dbConfig");
const PORT = 3000;

var app = express();
app.set("view engine", "ejs");
app.use(express.static(__dirname + '/public'));
app.use(bodyParser.urlencoded({ extended: true }));

app.use(session({
    secret: process.env.SESSION_SECRET,
    resave: false,
    saveUninitialized: false
})
);

app.use(passport.initialize());
app.use(passport.session());
app.use(flash());

app.get("/", (req, res) => {
    console.log(req.session.id_event);
    res.render("index");
});

app.get("/customers/registration", (req, res) => {
    res.render("customerregistration");
});

app.post("/customers/registration", (req, res) => {
    let { givenname, familyname, sex, birthday, email, password, address, city, postalcode, tel, cardamount } = req.body;
    var hashpassword = crypto.createHash('sha256').update(password + "kje6rE1j3s", 'utf8').digest('hex');
    pool.query("SELECT * FROM customers WHERE email = $1", [email], (err, results) => {
        if (err) { throw err; }
        if (results.rows.length > 0) {
            req.flash("failed_msg", "Email je již zaregistrován");
            return res.render("customerregistration");
        }
        else {
            pool.query("INSERT INTO addressbook (address, city, psc) VALUES ($1, $2, $3) RETURNING id_address", [address, city, postalcode], (err, results1) => {
                pool.query("INSERT INTO customers (name, surname, sex, birthday, email, password, phone, id_address) VALUES ($1, $2, $3, $4, $5, $6, $7, $8) RETURNING id_customers", [givenname, familyname, sex, birthday, email, hashpassword, tel, results1.rows[0].id_address], (err, results2) => {
                    pool.query("INSERT INTO orders (date, id_customer, cardamount, status) VALUES ($1, $2, $3, $4)", [moment().format(), results2.rows[0].id_customers, cardamount, false], (err, results3) => {
                        if (err) { throw err; }
                        console.log(results.rows);
                        req.flash("success_msg", "Byl jste úspěšně zaregistrován.");
                        res.redirect("/customers/login");
                    });
                });
            });
        }
    });
});


app.get("/:role/login", checkAuthenticated, (req, res) => {
    if (req.params.role == "users") {
        initializePassport(passport);
        req.session.role = "user";
        res.render("admin/adminlogin");
    }
    else if (req.params.role == "customers") {
        initializePassportCustomer(passport);
        req.session.role = "customer";
        res.render("customer/customerlogin");
    }
    console.log(req.session.role);
});

app.post("/customers/login", passport.authenticate("local", {
    successRedirect: "/customers/dashboard",
    failureRedirect: "/customers/login",
    failureFlash: true,
})
);

app.get("/customers/dashboard", checkNotAuthenticatedCustomer, (req, res) => {
    res.render("customer/customerdashboard", { user: req.user });
});

app.get("/users/dashboard", checkNotAuthenticatedUser, (req, res) => {
    pool.query("SELECT COUNT(id_order) FROM orders WHERE status = false", [], (err, results) => {
        pool.query("SELECT COALESCE(SUM(bills_order.amount*bills_order.price_w_vat) filter (where storno = false), 0) AS paysum, COALESCE(SUM(bills_order.amount*bills_order.price_w_vat) filter (where storno = true), 0) AS stornosum FROM bills_order JOIN bills ON bills_order.id_bill = bills.id_bill JOIN locations ON bills.id_location = locations.id_location JOIN events ON locations.id_event = events.id_events WHERE events.id_events = $1", [req.session.id_event], (err, results1) => {
            pool.query("SELECT COUNT(bills.id_bill),COUNT(bills.id_bill) filter (where storno = false) as stornofalse, COUNT(bills.id_bill) filter (where storno = true) as stornotrue FROM bills JOIN locations ON bills.id_location = locations.id_location JOIN events ON locations.id_event = events.id_events WHERE events.id_events = $1", [req.session.id_event], (err, results2) => {

                if (err) { throw err; }
                res.render("admin/admindashboard", { orders: results.rows[0].count, sales: results1.rows[0], bills: results2.rows[0], user: req.user });
            });
        });
    });
});

app.post("/users/login", passport.authenticate("local", {
    successRedirect: "/users/events/selectevent",
    failureRedirect: "/users/login",
    failureFlash: true
})
);

//#region Komponenty
app.get("/users/get/location", checkNotAuthenticatedUser, async (req, res) => {
    pool.query("SELECT * FROM locations WHERE id_event = $1", [req.session.id_event], (err, results) => {
        if (err) { throw err; }
        res.send({loc: results.rows});
    });
});
//#endregion

//#region Výběr Eventu
app.get("/users/events/selectevent", checkSelectEvents, (req, res) => {
    if (req.session.id_event > 0) {
        res.redirect("/users/dashboard");
    }
    else {
        moment.locale('cs');
        pool.query("SELECT * FROM events", [], (err, results) => {
            results.rows.map(function (date) {
                date.event_start = moment.unix(date.event_start / 1000).format('LLLL');
                date.event_end = moment.unix(date.event_end / 1000).format('LLLL');
            });
            if (err) { throw err; }
            res.render("admin/selectevent", { events: results.rows, user: req.user });
        });
    }
});

app.post("/users/selectevent/:id", checkSelectEvents, (req, res) => {
    req.session.id_event = req.params.id;
    res.redirect("/users/dashboard");
});

//#endregion

//#region Eventy
app.get("/users/events/add", checkNotAuthenticatedUser, (req, res) => {
    res.render("admin/addevent", { user: req.user });
});

var storage = multer.diskStorage({
    destination: function (req, file, cb) {
        cb(null, './public/img/events');
    },
    filename: function (req, file, cb) {
        cb(null, file.originalname);
    }
});

const upload = multer({
    storage: storage
}).single('image');

app.post("/users/events/add", upload, checkNotAuthenticatedUser, (req, res) => {
    let { name, start, end, location } = req.body;
    pool.query("INSERT INTO events (name, event_start, event_end, location, image) VALUES ($1, $2, $3, $4, $5)", [name, moment.utc(start).format(), moment.utc(end).format(), location, req.file.filename], (err, results) => {
        if (err) { throw err; }
        console.log(req.file.filename);
    });
    res.redirect(req.get('referer'));
});

app.get("/users/events/eventlist", checkNotAuthenticatedUser, (req, res) => {
    moment.locale('cs');
    pool.query("SELECT * FROM events", [], (err, results) => {
        results.rows.map(function (date) {
            date.event_start = moment.unix(date.event_start / 1000).format('LLLL');
            date.event_end = moment.unix(date.event_end / 1000).format('LLLL');
        });
        if (err) { throw err; }
        res.render("admin/eventlist", { events: results.rows, user: req.user });
    });
});

app.post("/users/events/selectevent/:id", checkNotAuthenticatedUser, (req, res) => {
    req.session.id_event = req.params.id;
    res.redirect(req.get('referer'));
});
//#endregion

//#region Produkty
app.get("/users/products", checkNotAuthenticatedUser, (req, res) => {
    pool.query("SELECT unit.code AS unitcode, dph.name AS dphname, products.code AS productscode, products.name AS productname, products_group.name AS productsgroupname,* FROM products JOIN dph ON products.id_dph = dph.id_dph JOIN products_group ON products.id_productsgroup = products_group.id_productsgroup JOIN unit ON products.id_unit = unit.id_unit WHERE products.id_event = $1", [req.session.id_event], (err, results) => {
        if (err) { throw err; }
        res.render("admin/products", { products: results.rows, user: req.user });
    });
});

app.get("/users/products/add", checkNotAuthenticatedUser, (req, res) => {
    pool.query("SELECT * FROM dph", [], (err, results) => {
        pool.query("SELECT * FROM products_group", [], (err, results1) => {
            pool.query("SELECT * FROM unit", [], (err, results2) => {
                if (err) { throw err; }
                res.render("admin/addproducts", { products: {}, units: results2.rows, productsgroup: results1.rows, dph: results.rows, user: req.user });
            });
        });
    });
});

app.post("/users/products/add", checkNotAuthenticatedUser, (req, res) => {
    let { code, name, value_n_vat, value_w_vat, text, id_dph, id_productsgroup, id_unit } = req.body;
    pool.query("INSERT INTO products (code, name, value_w_vat, value_n_vat, text, id_dph, id_productsgroup, id_unit, id_event) VALUES ($1, $2, $3, $4, $5, $6, $7, $8, $9)", [code, name, value_w_vat, value_n_vat, text, id_dph, id_productsgroup, id_unit, req.session.id_event], (err, results) => {
        if (err) { throw err; }
        res.redirect("/users/products");
    });
});

app.get("/users/products/edit/:id", checkNotAuthenticatedUser, (req, res) => {
    pool.query("SELECT * FROM dph", [], (err, results) => {
        pool.query("SELECT * FROM products WHERE id_products = $1", [req.params.id], (err, results1) => {
            pool.query("SELECT * FROM products_group", [], (err, results2) => {
                pool.query("SELECT * FROM unit", [], (err, results3) => {
                    if (err) { throw err; }
                    res.render("admin/editproducts", { units: results3.rows, productsgroup: results2.rows, products: results1.rows[0], dph: results.rows, user: req.user });
                });
            });
        });
    });
});

app.post("/users/products/edit/:id", checkNotAuthenticatedUser, (req, res) => {
    let { code, name, value_n_vat, value_w_vat, text, id_dph, id_productsgroup, id_unit } = req.body;
    pool.query("UPDATE products SET code = $1, name = $2, value_w_vat = $3, value_n_vat = $4, text = $5, id_dph = $6, id_productsgroup = $7, id_unit = $8 WHERE id_products = $9", [code, name, value_w_vat, value_n_vat, text, id_dph, id_productsgroup, id_unit, req.params.id], (err, results) => {
        if (err) { throw err; }
        res.redirect("/users/products");
    });
});

app.post("/users/products/delete/:id", checkNotAuthenticatedUser, (req, res) => {
    pool.query("DELETE FROM products WHERE id_products = $1", [req.params.id], (err, results) => {
        if (err) { throw err; }
        res.redirect(req.get('referer'));
    });
});
//#endregion

//#region Lokace
app.get("/users/location", checkNotAuthenticatedUser, (req, res) => {
    pool.query("SELECT * FROM locations WHERE id_event = $1", [req.session.id_event], (err, results) => {
        if (err) { throw err; }
        res.render("admin/locations", { locations: results.rows, user: req.user });
    });
});

app.post("/users/location/add", checkNotAuthenticatedUser, (req, res) => {
    pool.query("INSERT INTO locations (name, id_event) VALUES ($1, $2)", [req.body.location, req.session.id_event], (err, results) => {
        if (err) { throw err; }
        res.redirect(req.get('referer'));
    });
});

app.post("/users/location/delete/:id", checkNotAuthenticatedUser, (req, res) => {
    pool.query("DELETE FROM locations WHERE id_location = $1", [req.params.id], (err, results) => {
        if (err) { throw err; }
        res.redirect(req.get('referer'));
    });
});

app.post("/users/location/edit", checkNotAuthenticatedUser, (req, res) => {
    pool.query("UPDATE locations SET name = $1 WHERE id_location = $2", [req.body.location, req.body.id], (err, results) => {
        if (err) { throw err; }
        res.redirect(req.get('referer'));
    });
});
//#endregion

//#region Kategorie
app.get("/users/category", checkNotAuthenticatedUser, (req, res) => {
    pool.query("SELECT * FROM products_group WHERE id_event = $1", [req.session.id_event], (err, results) => {
        if (err) { throw err; }
        res.render("admin/category", { category: results.rows, user: req.user });
    });
});

app.post("/users/category/add", checkNotAuthenticatedUser, (req, res) => {
    pool.query("INSERT INTO products_group (name, id_event) VALUES ($1, $2)", [req.body.category, req.session.id_event], (err, results) => {
        if (err) { throw err; }
        res.redirect(req.get('referer'));
    });
});

app.post("/users/category/delete/:id", checkNotAuthenticatedUser, (req, res) => {
    pool.query("DELETE FROM products_group WHERE id_productsgroup = $1", [req.params.id], (err, results) => {
        if (err) { throw err; }
        res.redirect(req.get('referer'));
    });
});

app.post("/users/category/edit", checkNotAuthenticatedUser, (req, res) => {
    pool.query("UPDATE products_group SET name = $1 WHERE id_productsgroup = $2", [req.body.category, req.body.id], (err, results) => {
        if (err) { throw err; }
        res.redirect(req.get('referer'));
    });
});
//#endregion

//#region Jednotky
app.get("/users/unit", checkNotAuthenticatedUser, (req, res) => {
    pool.query("SELECT * FROM unit", [], (err, results) => {
        if (err) { throw err; }
        res.render("admin/units", { units: results.rows, user: req.user });
    });
});

app.post("/users/unit/add", checkNotAuthenticatedUser, (req, res) => {
    pool.query("INSERT INTO unit (name) VALUES ($1)", [req.body.unit], (err, results) => {
        if (err) { throw err; }
        res.redirect(req.get('referer'));
    });
});

app.post("/users/unit/delete/:id", checkNotAuthenticatedUser, (req, res) => {
    pool.query("DELETE FROM unit WHERE id_unit = $1", [req.params.id], (err, results) => {
        if (err) { throw err; }
        res.redirect(req.get('referer'));
    });
});

app.post("/users/unit/edit", checkNotAuthenticatedUser, (req, res) => {
    pool.query("UPDATE unit SET name = $1 WHERE id_unit = $2", [req.body.unit, req.body.id], (err, results) => {
        if (err) { throw err; }
        res.redirect(req.get('referer'));
    });
});
//#endregion

//#region Zákazníci
app.get("/users/customers", checkNotAuthenticatedUser, (req, res) => {
    pool.query("SELECT * FROM customers", [], (err, results) => {
        if (err) { throw err; }
        res.render("admin/customers", { customers: results.rows, user: req.user });
    });
});

app.get("/users/customers/:id", checkNotAuthenticatedUser, (req, res) => {
    pool.query("SELECT * FROM customers JOIN addressbook ON customers.id_address = addressbook.id_address WHERE customers.id_customers = $1", [req.params.id], (err, results) => {
        pool.query("SELECT TO_CHAR(date,'dd-mm-YYYY HH24:MI:SS') AS localdate, * FROM orders WHERE id_customer = $1", [req.params.id], (err, results1) => {
            pool.query("SELECT * FROM cards WHERE id_user = $1 ORDER BY id_card ASC", [req.params.id], (err, results2) => {
                if (err) { throw err; }
                results2.rows.map(function (card) {
                    card.cardnumber = card.cardnumber.replace(/\d{4}(?=.)/g, '$& ');
                });
                res.render("admin/customersdetail", { cards: results2.rows, orders: results1.rows, customers: results.rows[0], user: req.user });
            });
        });
    });
});

//#endregion

//#region Karty
Math.hRandom = function (length) {
    var number = "";
    var point = -1;
    for (let i = 0; i < length; i++) {
        if (i == point) number += ".";
        number += Math.floor(Math.random() * 10);
    }
    return number;
}

app.post("/users/customers/card/add", checkNotAuthenticatedUser, (req, res) => {
    var carduid = req.body.addcard;
    var cardsnumber;
    carduid.forEach(async uid => {

        do {
            var cardnumber = '2707' + Math.hRandom(12);
            cardsnumber = await pool.query("SELECT cardnumber FROM cards WHERE cardnumber = $1", [cardnumber]);
        }
        while (cardsnumber.rows.length = 0);

        pool.query("INSERT INTO cards (uid, credit, id_user, active, cardnumber) VALUES ($1, $2, $3, $4, $5)", [uid, 0, req.body.customerid, true, cardnumber], (err, results) => {
            if (err) { throw err; }
        });
    });
    res.redirect('back');
});

app.post("/users/customers/card/status/:id", checkNotAuthenticatedUser, (req, res) => {
    pool.query("SELECT active FROM cards WHERE id_card = $1", [req.params.id], (err, results) => {

        if (results.rows[0].active == false) {
            pool.query("UPDATE cards SET active = true WHERE id_card = $1", [req.params.id], (err, results1) => {
                if (err) { throw err; }
            });
        }
        else {
            pool.query("UPDATE cards SET active = false WHERE id_card = $1", [req.params.id], (err, results1) => {
                if (err) { throw err; }
            });
        }
    });
    res.redirect('back');
});

app.post("/users/customers/card/deposit/:id", checkNotAuthenticatedUser, (req, res) => {
    var cash = req.body.cash;
    if (cash > 0) {
        pool.query("UPDATE cards SET credit = credit + $1 WHERE id_card = $2", [cash, req.params.id], (err, results) => {
            if (err) { throw err; }
        });
    }

    res.redirect('back');
});
//#endregion

//#region Nastavení
app.get("/users/settings", checkNotAuthenticatedUser, (req, res) => {
    res.render("admin/settings", { user: req.user });
});

app.get("/customers/settings", checkNotAuthenticatedCustomer, (req, res) => {
    res.render("customer/settings", { user: req.user });
});
//#endregion

//#region Uživatele
app.get("/users/accounts", checkNotAuthenticatedUser, (req, res) => {
    pool.query("SELECT * FROM users", [], (err, results) => {
        if (err) { throw err; }
        res.render("admin/users", { users: results.rows, user: req.user });
    });
});

app.get("/users/accounts/add", checkNotAuthenticatedUser, (req, res) => {
    res.render("admin/addaccounts", { users: {}, user: req.user });
});

app.post("/users/accounts/add", checkNotAuthenticatedUser, (req, res) => {
    let { nickname, givenname, familyname, email, password, pin, sex, birthday, address, city, postalcode, tel } = req.body;
    var hashpassword = crypto.createHash('sha256').update(password + "kje6rE1j3s", 'utf8').digest('hex');
    pool.query("SELECT * FROM users WHERE email = $1 OR login = $2", [email, nickname], (err, results) => {
        if (err) { throw err; }
        if (results.rows.length > 0) {
            if (results.rows[0].email == email) {
                req.flash("failed_msg", "Email je již zaregistrován");
            }
            else if (results.rows[0].login == nickname) {
                req.flash("failed_msg", "Login je již zaregistrován");
            }
            res.redirect("/users/accounts/add");
        }
        else {
            pool.query("INSERT INTO addressbook (address, city, psc) VALUES ($1, $2, $3) RETURNING id_address", [address, city, postalcode], (err, results1) => {
                pool.query("INSERT INTO users (login, name, surname, sex, birthday, email, password, pin, phone, id_address) VALUES ($1, $2, $3, $4, $5, $6, $7, $8, $9, $10)", [nickname, givenname, familyname, sex, birthday, email, hashpassword, pin, tel, results1.rows[0].id_address], (err, results2) => {
                    if (err) { throw err; }
                    res.redirect("/users/accounts");
                });
            });
        }
    });
});

app.get("/users/accounts/edit/:id", checkNotAuthenticatedUser, (req, res) => {
    pool.query("SELECT TO_CHAR(birthday,'YYYY-mm-dd') AS localbirthday, * FROM users JOIN addressbook ON users.id_address = users.id_address WHERE id_user = $1", [req.params.id], (err, results) => {
        if (err) { throw err; }
        res.render("admin/editaccounts", { users: results.rows[0], user: req.user });
    });
});

//#endregion

//#region Objednávky
app.get("/users/orders", checkNotAuthenticatedUser, (req, res) => {
    pool.query("SELECT TO_CHAR(date,'dd-mm-YYYY HH24:MI:SS') AS localdate, * FROM orders", [], (err, results) => {
        if (err) { throw err; }
        res.render("admin/orders", { orders: results.rows, user: req.user });
    });
});

//#endregion

//#region Účtenky
app.get("/users/bills", checkNotAuthenticatedUser, (req, res) => {
    pool.query("SELECT SUM(bills_order.amount*bills_order.price_w_vat), bills.storno, bills.id_bill,created, users.name AS uname,users.surname AS usurname, customers.name AS cname,customers.surname AS csurname, locations.name AS locname, cards.uid FROM bills JOIN cards ON bills.id_card = cards.id_card JOIN users ON bills.id_user = users.id_user JOIN customers ON customers.id_customers = cards.id_user JOIN bills_order ON bills_order.id_bill = bills.id_bill JOIN products ON products.id_products = bills_order.id_product JOIN locations ON locations.id_location = bills.id_location WHERE locations.id_event = $1 GROUP BY bills.id_bill, users.name,users.surname, customers.name,customers.surname, locations.name, cards.uid ORDER BY bills.id_bill DESC", [req.session.id_event], (err, results) => {
        if (err) { throw err; }
        results.rows.map(function (date) {
            moment.locale('cs');
            date.created = moment.unix(date.created / 1000).format('LLL');
        });
        res.render("admin/bills", { bills: results.rows, user: req.user });
    });
});

app.get("/users/bills/:id", checkNotAuthenticatedUser, (req, res) => {
    pool.query("SELECT SUM(bills_order.amount*bills_order.price_w_vat), bills.storno, bills.id_bill,created, users.name,surname, locations.name AS locname, cards.uid FROM bills JOIN cards ON bills.id_card = cards.id_card JOIN users ON bills.id_user = users.id_user JOIN bills_order ON bills_order.id_bill = bills.id_bill JOIN products ON products.id_products = bills_order.id_product JOIN locations ON locations.id_location = bills.id_location WHERE bills.id_bill = $1 GROUP BY bills.id_bill, users.name,surname, locations.name, cards.uid ORDER BY bills.id_bill DESC", [req.params.id], (err, results) => {
        pool.query("SELECT *, price_w_vat*amount AS totalprice FROM bills_order JOIN products ON bills_order.id_product = products.id_products JOIN unit ON products.id_unit = unit.id_unit WHERE bills_order.id_bill = $1", [req.params.id], (err, results1) => {
            if (err) { throw err; }
            results.rows.map(function (date) {
                moment.locale('cs');
                date.created = moment.unix(date.created / 1000).format('LLLL');
            });
            res.render("admin/billsdetail", { bills: results.rows[0], billsorder: results1.rows, user: req.user });
        });
    });
});
//#endregion

//#region Customer Účtenky
app.get("/customers/bills", checkNotAuthenticatedCustomer, (req, res) => {
    pool.query("SELECT SUM(bills_order.amount*bills_order.price_w_vat), bills.storno, bills.id_bill,created, users.name,surname, locations.name AS locname, cards.cardnumber FROM bills JOIN cards ON bills.id_card = cards.id_card JOIN users ON bills.id_user = users.id_user JOIN bills_order ON bills_order.id_bill = bills.id_bill JOIN products ON products.id_products = bills_order.id_product JOIN locations ON locations.id_location = bills.id_location WHERE cards.id_user = $1 GROUP BY bills.id_bill, users.name,surname, locations.name, cards.cardnumber ORDER BY bills.id_bill DESC", [req.user.id_customers], (err, results) => {
        if (err) { throw err; }
        results.rows.map(function (date) {
            moment.locale('cs');
            date.created = moment.unix(date.created / 1000).format('LLL');
        });
        res.render("customer/bills", { bills: results.rows, user: req.user });
    });
});

app.get("/customers/bills/:id", checkNotAuthenticatedCustomer, (req, res) => {
    pool.query("SELECT SUM(bills_order.amount*bills_order.price_w_vat), bills.storno, bills.id_bill,created, users.name,surname, locations.name AS locname, cards.uid FROM bills JOIN cards ON bills.id_card = cards.id_card JOIN users ON bills.id_user = users.id_user JOIN bills_order ON bills_order.id_bill = bills.id_bill JOIN products ON products.id_products = bills_order.id_product JOIN locations ON locations.id_location = bills.id_location WHERE bills.id_bill = $1 AND cards.id_user = $2 GROUP BY bills.id_bill, users.name,surname, locations.name, cards.uid ORDER BY bills.id_bill DESC", [req.params.id, req.user.id_customers], (err, results) => {
        pool.query("SELECT *, price_w_vat*amount AS totalprice FROM bills_order JOIN products ON bills_order.id_product = products.id_products JOIN unit ON products.id_unit = unit.id_unit WHERE bills_order.id_bill = $1", [req.params.id], (err, results1) => {
            if (err) { throw err; }
            if (results.rows.length > 0) {
                results.rows.map(function (date) {
                    moment.locale('cs');
                    date.created = moment.unix(date.created / 1000).format('LLLL');
                });
                res.render("customer/billsdetail", { bills: results.rows[0], billsorder: results1.rows, user: req.user });
            }
            else {
                res.status(401).send;
            }
        });
    });
});
//#endregion

//#region Customer Karty
app.get("/customers/cards", checkNotAuthenticatedCustomer, (req, res) => {
    pool.query("SELECT * FROM cards WHERE id_user = $1 ORDER BY id_card ASC", [req.user.id_customers], (err, results) => {
        if (err) { throw err; }
        results.rows.map(function (card) {
            card.cardnumber = card.cardnumber.replace(/\d{4}(?=.)/g, '$& ');
        });
        res.render("customer/cards", { cards: results.rows, user: req.user });
    });
});
//#endregion

//#region Statistiky
//#region Prodeje
app.get("/users/stats/sales", checkNotAuthenticatedUser, (req, res) => {
    var from = req.query.from ? from = req.query.from : from = moment().startOf('isoweek').format('YYYY-MM-DD 00:00:00')
    var to = req.query.to ? to = req.query.to : to = moment().endOf('isoweek').format('YYYY-MM-DD 23:59:59')
    var loc = req.query.loc && req.query.loc > 0 ? loc = `AND locations.id_location = ${req.query.loc}` : loc = ''
    pool.query(`SELECT products.name, bills_order.price_w_vat, bills_order.amount, events.name AS event_name, locations. name AS loc_name, bills.id_bill, created FROM bills_order
    JOIN products ON bills_order.id_product = products.id_products
    JOIN bills ON bills_order.id_bill = bills.id_bill
    JOIN locations ON bills.id_location = locations.id_location
    JOIN events ON locations.id_event = events.id_events
    WHERE id_events = $1 AND storno = false AND created BETWEEN '${from}' AND '${to}' ${loc}
    GROUP BY products.id_products, bills_order.id_billorder, bills.id_bill, events.name, locations.name`, [req.session.id_event], (err, results) => {
        if (err) { throw err; }
        res.render("admin/stats/sales/sales", { sales: results.rows, odkdy: from, dokdy: to, user: req.user });
    });
});

app.get("/users/stats/salesbyitem", checkNotAuthenticatedUser, (req, res) => {
    var from = req.query.from ? from = req.query.from : from = moment().startOf('isoweek').format('YYYY-MM-DD 00:00:00')
    var to = req.query.to ? to = req.query.to : to = moment().endOf('isoweek').format('YYYY-MM-DD 23:59:59')
    var loc = req.query.loc && req.query.loc > 0 ? loc = `AND locations.id_location = ${req.query.loc}` : loc = ''
    pool.query(`SELECT products.name, SUM(bills_order.price_w_vat * bills_order.amount) AS price_w_vat, SUM(bills_order.amount) AS amount, events.name AS event_name, locations. name AS loc_name FROM bills_order
    JOIN products ON bills_order.id_product = products.id_products
    JOIN bills ON bills_order.id_bill = bills.id_bill
    JOIN locations ON bills.id_location = locations.id_location
    JOIN events ON locations.id_event = events.id_events
    WHERE id_events = $1 AND storno = false AND created BETWEEN '${from}' AND '${to}' ${loc}
    GROUP BY products.id_products, events.name, locations.name`, [req.session.id_event], (err, results) => {
        if (err) { throw err; }
        res.render("admin/stats/sales/salesbyitem", { sales: results.rows, odkdy: from, dokdy: to, user: req.user });
    });
});

app.get("/users/stats/salesbydays", checkNotAuthenticatedUser, (req, res) => {
    var from = req.query.from ? from = req.query.from : from = moment().startOf('isoweek').format('YYYY-MM-DD 00:00:00')
    var to = req.query.to ? to = req.query.to : to = moment().endOf('isoweek').format('YYYY-MM-DD 23:59:59')
    var loc = req.query.loc && req.query.loc > 0 ? loc = `AND locations.id_location = ${req.query.loc}` : loc = ''
    pool.query(`SELECT t1.week_day, t2.sales_count, t2.loc_name, t2.event_name
    FROM (SELECT day, To_char(day, 'DD.MM.YYYY - TMDay') week_day
    FROM generate_series('${from}'::DATE, '${to}'::DATE, '1 day'::interval) AS day) t1
    LEFT OUTER JOIN (SELECT To_char(created, 'DD.MM.YYYY - TMDay') week_day, SUM(amount) AS sales_count, locations.name AS loc_name, events.name AS event_name
    FROM (bills_order JOIN products ON bills_order.id_product = products.id_products
    JOIN bills ON bills_order.id_bill = bills.id_bill
    JOIN locations ON bills.id_location = locations.id_location
    JOIN events ON locations.id_event = events.id_events)
    WHERE id_events = $1 AND storno = false AND created BETWEEN '${from}' AND '${to}' ${loc}
    GROUP  BY week_day, loc_name, event_name) t2
    ON t1.week_day = t2.week_day
    ORDER BY day;`, [req.session.id_event], (err, results) => {
        if (err) { throw err; }
        res.render("admin/stats/sales/salesbyday", { sales: results.rows, odkdy: from, dokdy: to, user: req.user });
    });
});

app.get("/users/stats/salesbyhours", checkNotAuthenticatedUser, (req, res) => {
    var from = req.query.from ? from = req.query.from : from = moment().startOf('isoweek').format('YYYY-MM-DD 00:00:00')
    var to = req.query.to ? to = req.query.to : to = moment().endOf('isoweek').format('YYYY-MM-DD 23:59:59')
    var loc = req.query.loc && req.query.loc > 0 ? loc = `AND locations.id_location = ${req.query.loc}` : loc = ''
    pool.query(`SELECT t1.week_day, t2.sales_count, t2.loc_name, t2.event_name
    FROM (SELECT day, To_char(day, 'DD.MM.YYYY - TMDay HH24') week_day
    FROM generate_series('${from}'::DATE, '${to}'::DATE, '1 hour'::interval) AS day) t1
    LEFT OUTER JOIN (SELECT To_char(created, 'DD.MM.YYYY - TMDay HH24') week_day, SUM(amount) AS sales_count, locations.name AS loc_name, events.name AS event_name
    FROM (bills_order JOIN products ON bills_order.id_product = products.id_products
    JOIN bills ON bills_order.id_bill = bills.id_bill
    JOIN locations ON bills.id_location = locations.id_location
    JOIN events ON locations.id_event = events.id_events)
    WHERE id_events = $1 AND storno = false AND created BETWEEN '${from}' AND '${to}' ${loc}
    GROUP  BY week_day, loc_name, event_name) t2
    ON t1.week_day = t2.week_day
    ORDER BY day;`, [req.session.id_event], (err, results) => {
        if (err) { throw err; }
        res.render("admin/stats/sales/salesbyday", { sales: results.rows, odkdy: from, dokdy: to, user: req.user });
    });
});
//#endregion

//#region Storno
app.get("/users/stats/storno", checkNotAuthenticatedUser, (req, res) => {
    var from = req.query.from ? from = req.query.from : from = moment().startOf('isoweek').format('YYYY-MM-DD 00:00:00')
    var to = req.query.to ? to = req.query.to : to = moment().endOf('isoweek').format('YYYY-MM-DD 23:59:59')
    var loc = req.query.loc && req.query.loc > 0 ? loc = `AND locations.id_location = ${req.query.loc}` : loc = ''
    pool.query(`SELECT products.name, bills_order.price_w_vat, bills_order.amount, events.name AS event_name, locations. name AS loc_name, bills.id_bill, created FROM bills_order
    JOIN products ON bills_order.id_product = products.id_products
    JOIN bills ON bills_order.id_bill = bills.id_bill
    JOIN locations ON bills.id_location = locations.id_location
    JOIN events ON locations.id_event = events.id_events
    WHERE id_events = $1 AND storno = true AND created BETWEEN '${from}' AND '${to}' ${loc}
    GROUP BY products.id_products, bills_order.id_billorder, bills.id_bill, events.name, locations.name`, [req.session.id_event], (err, results) => {
        if (err) { throw err; }
        res.render("admin/stats/storno/storno", { sales: results.rows, odkdy: from, dokdy: to, user: req.user });
    });
});

app.get("/users/stats/stornobyitem", checkNotAuthenticatedUser, (req, res) => {
    var from = req.query.from ? from = req.query.from : from = moment().startOf('isoweek').format('YYYY-MM-DD 00:00:00')
    var to = req.query.to ? to = req.query.to : to = moment().endOf('isoweek').format('YYYY-MM-DD 23:59:59')
    var loc = req.query.loc && req.query.loc > 0 ? loc = `AND locations.id_location = ${req.query.loc}` : loc = ''
    pool.query(`SELECT products.name, SUM(bills_order.price_w_vat * bills_order.amount) AS price_w_vat, SUM(bills_order.amount) AS amount, events.name AS event_name, locations. name AS loc_name FROM bills_order
    JOIN products ON bills_order.id_product = products.id_products
    JOIN bills ON bills_order.id_bill = bills.id_bill
    JOIN locations ON bills.id_location = locations.id_location
    JOIN events ON locations.id_event = events.id_events
    WHERE id_events = $1 AND storno = true AND created BETWEEN '${from}' AND '${to}' ${loc}
    GROUP BY products.id_products, events.name, locations.name`, [req.session.id_event], (err, results) => {
        if (err) { throw err; }
        res.render("admin/stats/storno/stornobyitem", { sales: results.rows, odkdy: from, dokdy: to, user: req.user });
    });
});

app.get("/users/stats/stornobydays", checkNotAuthenticatedUser, (req, res) => {
    var from = req.query.from ? from = req.query.from : from = moment().startOf('isoweek').format('YYYY-MM-DD 00:00:00')
    var to = req.query.to ? to = req.query.to : to = moment().endOf('isoweek').format('YYYY-MM-DD 23:59:59')
    var loc = req.query.loc && req.query.loc > 0 ? loc = `AND locations.id_location = ${req.query.loc}` : loc = ''
    pool.query(`SELECT t1.week_day, t2.sales_count, t2.loc_name, t2.event_name
    FROM (SELECT day, To_char(day, 'DD.MM.YYYY - TMDay') week_day
    FROM generate_series('${from}'::DATE, '${to}'::DATE, '1 day'::interval) AS day) t1
    LEFT OUTER JOIN (SELECT To_char(created, 'DD.MM.YYYY - TMDay') week_day, SUM(amount) AS sales_count, locations.name AS loc_name, events.name AS event_name
    FROM (bills_order JOIN products ON bills_order.id_product = products.id_products
    JOIN bills ON bills_order.id_bill = bills.id_bill
    JOIN locations ON bills.id_location = locations.id_location
    JOIN events ON locations.id_event = events.id_events)
    WHERE id_events = $1 AND storno = true AND created BETWEEN '${from}' AND '${to}' ${loc}
    GROUP  BY week_day, loc_name, event_name) t2
    ON t1.week_day = t2.week_day
    ORDER BY day;`, [req.session.id_event], (err, results) => {
        if (err) { throw err; }
        res.render("admin/stats/storno/stornobyday", { sales: results.rows, odkdy: from, dokdy: to, user: req.user });
    });
});

app.get("/users/stats/stornobyhours", checkNotAuthenticatedUser, (req, res) => {
    var from = req.query.from ? from = req.query.from : from = moment().startOf('isoweek').format('YYYY-MM-DD 00:00:00')
    var to = req.query.to ? to = req.query.to : to = moment().endOf('isoweek').format('YYYY-MM-DD 23:59:59')
    var loc = req.query.loc && req.query.loc > 0 ? loc = `AND locations.id_location = ${req.query.loc}` : loc = ''
    pool.query(`SELECT t1.week_day, t2.sales_count, t2.loc_name, t2.event_name
    FROM (SELECT day, To_char(day, 'DD.MM.YYYY - TMDay HH24') week_day
    FROM generate_series('${from}'::DATE, '${to}'::DATE, '1 hour'::interval) AS day) t1
    LEFT OUTER JOIN (SELECT To_char(created, 'DD.MM.YYYY - TMDay HH24') week_day, SUM(amount) AS sales_count, locations.name AS loc_name, events.name AS event_name
    FROM (bills_order JOIN products ON bills_order.id_product = products.id_products
    JOIN bills ON bills_order.id_bill = bills.id_bill
    JOIN locations ON bills.id_location = locations.id_location
    JOIN events ON locations.id_event = events.id_events)
    WHERE id_events = $1 AND storno = true AND created BETWEEN '${from}' AND '${to}' ${loc}
    GROUP  BY week_day, loc_name, event_name) t2
    ON t1.week_day = t2.week_day
    ORDER BY day;`, [req.session.id_event], (err, results) => {
        if (err) { throw err; }
        res.render("admin/stats/storno/stornobyday", { sales: results.rows, odkdy: from, dokdy: to, user: req.user });
    });
});
//#endregion

//#region tržby
app.get("/users/stats/takes", checkNotAuthenticatedUser, (req, res) => {
    var from = req.query.from ? from = req.query.from : from = moment().startOf('isoweek').format('YYYY-MM-DD 00:00:00')
    var to = req.query.to ? to = req.query.to : to = moment().endOf('isoweek').format('YYYY-MM-DD 23:59:59')
    var loc = req.query.loc && req.query.loc > 0 ? loc = `AND locations.id_location = ${req.query.loc}` : loc = ''
    pool.query(`SELECT products.name, bills_order.price_w_vat, bills_order.amount, events.name AS event_name, locations. name AS loc_name, bills.id_bill, created FROM bills_order
    JOIN products ON bills_order.id_product = products.id_products
    JOIN bills ON bills_order.id_bill = bills.id_bill
    JOIN locations ON bills.id_location = locations.id_location
    JOIN events ON locations.id_event = events.id_events
    WHERE id_events = $1 AND storno = false AND created BETWEEN '${from}' AND '${to}' ${loc}
    GROUP BY products.id_products, bills_order.id_billorder, bills.id_bill, events.name, locations.name`, [req.session.id_event], (err, results) => {
        if (err) { throw err; }
        res.render("admin/stats/takes/takes", { sales: results.rows, odkdy: from, dokdy: to, user: req.user });
    });
});
//#endregion

//#endregion

app.get("/users/logout", (req, res) => {
    req.logout();
    delete req.session.id_event;
    res.render("admin/adminlogin");
});

app.get("/customers/logout", (req, res) => {
    req.logout();
    res.render("customer/customerlogin");
});

app.get("/users/about", checkNotAuthenticatedUser, (req, res) => {
    res.render("admin/about", { user: req.user });
});

app.get("/customers/about", checkNotAuthenticatedCustomer, (req, res) => {
    res.render("customer/about", { user: req.user });
});

app.get("/users/account", checkNotAuthenticatedUser, (req, res) => {
    pool.query("SELECT TO_CHAR(birthday,'dd. mm. YYYY') AS localbirthday, * FROM users JOIN addressbook ON users.id_address = addressbook.id_address WHERE users.id_user = $1", [req.user.id_user], (err, results) => {
        if (err) { throw err; }
        res.render("admin/accountinfo", { account: results.rows[0], user: req.user });
    });
});

function checkSelectEvents(req, res, next) {
    if (req.isAuthenticated() && req.session.role == "user") {
        if (!req.session.id_event > 0) {
            return next();
        }

    }
    res.redirect("/users/login");
}

function checkAuthenticated(req, res, next) {
    if (req.isAuthenticated() && req.session.role == "user") {
        if (req.session.id_event > 0) {
            return res.redirect("/users/dashboard");
        }
        else {
            return res.redirect("/users/events/selectevent");
        }
    }
    else if (req.isAuthenticated() && req.session.role == "customer") {
        return res.redirect("/customers/dashboard");
    }
    next();
}

function checkNotAuthenticatedUser(req, res, next) {
    if (req.isAuthenticated() && req.session.role == "user") {
        if (!req.session.id_event > 0) {
            res.redirect("/users/events/selectevent");
        }
        return next();

    }
    res.redirect("/users/login");
}

function checkNotAuthenticatedCustomer(req, res, next) {
    if (req.isAuthenticated() && req.session.role == "customer") {
        return next();
    }
    res.redirect("/customers/login");
}

app.listen(PORT, () => {
    console.log(`Server running on port ${PORT}`);
});